function [kernel_pdf]=gaussiankernel(obs,nb_points,bandwidth)
% kernel_pdf=gaussiankernel(obs,nb_points,bandwidth)
% 
% The function gaussiankernel estimates the density of the distribution 
% underlying the sample with the non-parametric gaussian kernel method.  

%
% Input:
% - obs is a vector of observations 
% - nb_points is the number of points where the density is estimated (see the remark) 
% - bandwidth is an optional argument
%
% Output:
% - kernel_pdf is a matrix (2,nbpoints) where the first row contains the 
%   estimation points and the second row contains the values of the density. 
%
% Remark:
% The density is estimated between the min and the max of the sample of
% observations, for nb_points equidistant points.
%

% Remark: all the above comments on the function will be displayed by Matlab 
% when we type help gaussiankernel in the command window.

nb_obs=length(obs);
% the following calculations require obs to be a column vector.

obs=reshape(obs,nb_obs,1);
% The 3rd input argument of the function gaussiankernel is optional.
% If no value is passed for the bandwith, the program use the "rule of
% thumb".


if nargin==2
    bandwidth=(nb_obs^(-0.2))*(var(obs)^0.5);
end
% Creation of the grid of points
max_obs=max(obs);
min_obs=min(obs);
step=(max_obs-min_obs)/(nb_points-1);
xx=[min_obs:step:max_obs];
% We calculate all the points in one step by using the functions mean and
% the kronecker product.
% This is the most compact and efficient writing as
% it optimize the use of the preprogrammed functions of Matlab.
% Matlab is an interpreted language (as opposed to compiled language), you
% should avoid the use of loops and you should "vectorize" the code at best
% to obtain a good performance.
% Check the help on different functions for more details.

kernel_pdf=[xx; mean(exp(-0.5*((...
    kron(ones(nb_obs,1),xx)-kron(obs,ones(1,nb_points))...
    )/bandwidth).^2),1)/bandwidth/((2*pi)^0.5)];

